

#include "preprocess.h"
#include <algorithm>
//using namespace concurrency;
Vec3D<float> FCos3dPreProcessor::Process(std::vector<uchar> *imageVectorPtr) {

// cv::Mat imageMat(height, width, CV_8UC3, imageVectorPtr->data());
        cv::Mat imageMat = vectorPointerToMat(imageVectorPtr,
                                              fcos3DPreProcessParameter.height,
                                              fcos3DPreProcessParameter.width);
// std::cout << "image mat inside the preprocess: " << std::endl << imageMat << std::endl;


        cv::Mat floatImage;
        imageMat.convertTo(floatImage, CV_32FC3);
        // Split the image into separate channels
        std::vector<cv::Mat> channels;
        cv::split(floatImage, channels);
//    tbb::parallel_for(size_t(0), size_t(channels.size()), [&] (size_t i) {
//        channels[i].convertTo(channels[i], CV_32FC1);
//        channels[i] = (channels[i] - fcos3DPreProcessParameter.mean[i]) / fcos3DPreProcessParameter.std_dev[i];
//    });

// Normalize each channel independently
        for (int i = 0; i < floatImage.channels(); ++i) {
            channels[i].convertTo(channels[i], CV_32FC1);
            channels[i] = (channels[i] - fcos3DPreProcessParameter.mean[i]) / fcos3DPreProcessParameter.std_dev[i];
        }

// Merge the channels back into the original image
        cv::merge(channels, floatImage);

//return pad_image_Mat(floatImage);
        return image_vector_without_padding(floatImage);
    }

bool FCos3dPreProcessor::Init(const Fcos3dConstantParameter &init_params) {
    this->fcos3DPreProcessParameter.mean = init_params.mean;
    this->fcos3DPreProcessParameter.std_dev = init_params.std_dev;
    this->fcos3DPreProcessParameter.width = init_params.width;
    this->fcos3DPreProcessParameter.height = init_params.height;
    this->fcos3DPreProcessParameter.channel = init_params.channel;
    this->fcos3DPreProcessParameter.num_threads = init_params.num_threads;
    return true;
}

std::vector<uchar>* FCos3dPreProcessor::matToVectorPointer(const cv::Mat &mat) {
    if (mat.empty()) {
        return nullptr; // Return nullptr if the input matrix is empty
    }

    // Calculate the total number of elements in the cv::Mat
    int totalSize = mat.total() * mat.elemSize();

    // Create a new vector with the same data as the cv::Mat
    std::vector<uchar> *vec = new std::vector<uchar>(mat.data, mat.data + totalSize);

    return vec;
}

cv::Mat FCos3dPreProcessor::vectorPointerToMat(const std::vector<uchar> *vec, int rows, int cols) {
    // Create a cv::Mat with the specified rows and cols
    cv::Mat temp(rows, cols, CV_8UC3);
    // Copy the data from the vector to the cv::Mat
    memcpy(temp.data, vec->data(), rows * cols * 3 * sizeof(unsigned char));
    return temp;
}

Vec3D<float> FCos3dPreProcessor::image_vector_without_padding(cv::Mat img) {
//    Vec3D<float> img_new(img.channels(), std::vector<std::vector<float>>(img.rows, std::vector<float>(img.cols)));
    Vec3D<float> img_new(img.channels(), std::vector<std::vector<float>>(img.rows, std::vector<float>(img.cols)));
    for (int i = 0; i < img.rows; ++i) {
        for (int j = 0; j < img.cols; ++j) {
            cv::Vec3f pixel = img.at<cv::Vec3f>(i, j);
            for (int k = 0; k < img.channels(); ++k) {
                img_new[k][i][j] = pixel[k];
            }
        }
    }
    return img_new;


}


Vec3D<float> FCos3dPreProcessor::image_vector_without_padding_channel_last(cv::Mat img){
//    Vec3D<float> img_new(img.channels(), std::vector<std::vector<float>>(img.rows, std::vector<float>(img.cols)));
    Vec3D<float> img_new(1600,std::vector<std::vector<float>>(900, std::vector<float>(img.channels())));
    for (int i = 0; i < 1600; ++i) {
        for (int j = 0; j < 900; ++j) {
            cv::Vec3f pixel = img.at<cv::Vec3f>(i, j);
            for (int k = 0; k < img.channels(); ++k) {
                img_new[i][j][k] = pixel[k];
            }
        }
    }
    return img_new;


}